"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const child = require("child_process");
const os = require("os");
const path = require("path");
class ParallelMapReduce {
    static getInstance() {
        if (!ParallelMapReduce.instance) {
            ParallelMapReduce.instance = new ParallelMapReduce();
        }
        return ParallelMapReduce.instance;
    }
    mapReduce(data, mapFn, reduceFn, options) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!data.length) {
                return Promise.reject(new Error('Data is empty'));
            }
            const dataClone = data.slice();
            options = Object.assign({
                workers: os.cpus().length
            }, options);
            let mapRsp = [];
            try {
                mapRsp = yield ParallelMapReduce.getInstance().map(dataClone, mapFn, options);
            }
            catch (err) {
                return Promise.reject(err);
            }
            let reduceRsp = null;
            try {
                reduceRsp = yield ParallelMapReduce.getInstance().reduce(mapRsp, reduceFn);
            }
            catch (err) {
                return Promise.reject(err);
            }
            return Promise.resolve(reduceRsp);
        });
    }
    map(data, fn, options) {
        return new Promise((resolve, reject) => {
            const promises = [];
            const dataSplitIndex = Math.floor(data.length / options.workers);
            const fnString = fn.toString();
            for (let x = 1; x <= options.workers; x++) {
                const spliceIndex = (x === options.workers ? data.length : dataSplitIndex);
                const splicedData = data.splice(0, spliceIndex);
                const wp = {
                    id: x,
                    fn: fnString,
                    data: splicedData,
                };
                promises.push(this.spawnWorker(wp, fn));
            }
            Promise.all(promises)
                .then((rsp) => {
                return resolve(rsp);
            })
                .catch((err) => {
                return reject(err);
            });
        });
    }
    reduce(data, fn) {
        return new Promise((resolve, reject) => {
            const wp = {
                id: 1,
                fn: fn.toString(),
                data: data,
            };
            return this.spawnWorker(wp, fn)
                .then((rsp) => {
                return resolve(rsp);
            })
                .catch((err) => {
                return reject(err);
            });
        });
    }
    spawnWorker(payload, fn) {
        return new Promise((resolve, reject) => {
            const subProcess = child.fork(ParallelMapReduce.workerPath, [], {});
            subProcess.send(payload);
            subProcess.on('message', (pl) => {
                return resolve(pl);
            });
            subProcess.on('error', error => {
                return reject(error);
            });
        });
    }
}
ParallelMapReduce.instance = null;
ParallelMapReduce.workerPath = path.resolve(`${__dirname}/Worker.js`);
exports.ParallelMapReduce = ParallelMapReduce;
